@@ -13,21 +13,30 @@ except ImportError:
 	from __version__ import __version__
 
 
+def isUTF8(file):
+	with open(file, 'rb') as f:
+		data = f.read()
+	try:
+		decoded = data.decode('UTF-8')
+	except UnicodeDecodeError:
+		return False
+	else:
+		for ch in decoded:
+			if 0xD800 <= ord(ch) <= 0xDFFF:
+				return False
+		return True
+
+
 def transpile(infile, outfile="stdout", decompileFile=True, forceDecompile=False, multiplication=True, run=False):
 
-	decode = os.path.splitext(infile)[1].lower() == ".8xp" and decompileFile is True
+	decode = not isUTF8(infile) and decompileFile is True
 
 	if decode is True or forceDecompile is True:
 		# Decompile 8Xp file
-		try:
-			temp_name = next(tempfile._get_candidate_names())
-			while os.path.exists(temp_name):
-				temp_name = next(tempfile._get_candidate_names())
-			btb.decompile_file(infile, temp_name)
-			with open(temp_name, 'r') as f:
-				pythonCode = TIBasicParser([line.strip() for line in f.readlines()], multiplication).toPython()
-		finally:
-			os.remove(temp_name)
+		with tempfile.NamedTemporaryFile() as f:
+			btb.decompile_file(infile, f.name)
+			with open(f.name, 'r') as fp:
+				pythonCode = TIBasicParser([line.strip() for line in fp.readlines()], multiplication).toPython()
 
 	else:
 		# Dont decompile
@@ -36,7 +45,7 @@ def transpile(infile, outfile="stdout", decompileFile=True, forceDecompile=False
 		with open(infile, 'r') as f:
 			file_lines = [line.strip() for line in f.readlines()]
 
-		pythonCode = TIBasicParser(file_lines).toPython()
+		pythonCode = TIBasicParser(file_lines, multiplication).toPython()
 
 	# Write to outfile
 	if outfile == "stdout":
