@@ -3,103 +3,17 @@ import logging
 import os
 import shutil
 
+try:
+	from . import formatting
+except ImportError:
+	import formatting
+
 logger = logging.getLogger(__name__)
 logging.basicConfig(level=logging.WARNING)
 
 
-def closeOpen(string):
-	if string.count("\"") % 2 != 0:
-		string = string + "\""
-
-	# Split string by "
-	splitString = string.split('"')
-	open = 0
-	closed = 0
-
-	for i in range(0, len(splitString), 2): # skip all even elements since those are in between quotes
-		open += splitString[i].count('(')
-		closed += splitString[i].count(')')
-
-	# Add needed closing parentheses
-	string = string + ')' * (open - closed)
-	return string
-
-
-def menu(title, args):
-	choices = []
-	for i in range(0, len(args), 2):
-		choices.append((str((i // 2) + 1) + ".", str(args[i])))
-
-	# Convert args to dict
-	args = {str((i // 2) + 1) + ".": args[i + 1] for i in range(0, len(args), 2)}
-
-	ifstmt = ["d = dialog.Dialog()", "menu = d.menu(\"" + title + "\", choices=" + str(choices) + ")"]
-	for key in args:
-		ifstmt.extend(["if menu[1] == \"" + str(key) + "\":", "\tclear()", "\tgoto .lbl" + str(args[key])])
-
-	return ifstmt
-
-
-def fixEquals(string):
-	# Change = to == if not between quotes
-	return re.sub(r'(?!\B"[^"]*)(?<![<>!])\=(?!\()+(?![^"]*"\B)', " == ", string)
-
-
-def noStringReplace(regexp, newSubstr, statementList):
-	# Replace everything in statementList that matches regexp with newSubstr, as long as its not in a string
-	# Split by "
-	statementList = [item.split('"') for item in statementList]
-
-	# Iterate through expression list
-	for i in range(len(statementList)):
-		# Skip all even elements since those are in between quotes
-		for j in range(0, len(statementList[i]), 2):
-			statementList[i][j] = re.sub(regexp, newSubstr, statementList[i][j])
-
-		# Restore list element
-		statementList[i] = '"'.join(statementList[i])
-
-	return statementList
-
-
-def parenthesis_split(sentence, separator=" ", lparen="(", rparen=")"):
-	nb_brackets = 0
-	sentence = sentence.strip(separator) # get rid of leading/trailing seps
-
-	l = [0]
-	for i, c in enumerate(sentence):
-		if c == lparen:
-			nb_brackets += 1
-		elif c == rparen:
-			nb_brackets -= 1
-		elif c == separator and nb_brackets == 0:
-			l.append(i)
-		# handle malformed string
-		if nb_brackets < 0:
-			raise Exception("Syntax error")
-
-	l.append(len(sentence))
-	# handle missing closing parentheses
-	if nb_brackets > 0:
-		raise Exception("Syntax error")
-
-	return([sentence[i:j].strip(separator) for i, j in zip(l, l[1:])])
-
-
-def toValidEqn(eqn):
-	# Try to turn things like AB into A*B and A(3) into A*(3)
-	eqn = eqn.split('"')
-	for i in range(0, len(eqn), 2): # Skip all even elements since those are in between quotes
-		while re.search(r'((?:[A-Z0-9]+)|(?:[A-Z]\w*\(\w+\)))((?:[A-Z]\w*)|\()', eqn[i]) or re.search(r'(\))((?:[A-Z]\w*)|\()', eqn[i]):
-			eqn[i] = re.sub(r"((?:[A-Z0-9]+)|(?:[A-Z]\w*\(\w+\)))((?:[A-Z]\w*)|\()", r"\1*\2", eqn[i])
-			eqn[i] = re.sub(r'(\))((?:[A-Z]\w*)|\()', r'\1*\2', eqn[i])
-	eqn = '"'.join(eqn)
-	return eqn
-
-
 class TIBasicParser(object):
 	def __init__(self, basic, multiplication):
-		# if multiplication is True, the transpiler will attempt to fix implicit multiplication
 		if isinstance(basic, list):
 			self.basic = basic
 		elif isinstance(basic, str):
@@ -123,14 +37,15 @@ class TIBasicParser(object):
 		self.drawLock = False
 
 	def convertLine(self, index, line):
-		if self.multiplication:
-			line = toValidEqn(line)
+		# TODO: possible curses interface instead of just printing and outputting
 
 		statement = ""
+
+		# Fix lists
+		number_table = {'₁': '1', '₂': '2', '₃': '3', '₄': '4', '₅': '5', '₆': '6'}
+		line = re.sub('(?:L|l)([₁₂₃₄₅₆]){1}', lambda m: 'L' + number_table[m.group(1)], line)
+
 		# TODO: Make rules for :, dont fully understand it yet
-		# if line.startswith("\""):
-			# Comments
-			# statement = "# " + line.lstrip("\"")
 		if self.skipLine > 0:
 			self.skipLine -= 1
 			return None
@@ -140,7 +55,7 @@ class TIBasicParser(object):
 		# Disp
 		elif line.startswith("Disp "):
 			statement = re.search("Disp (.*[^)])", line).groups(1)[0]
-			statement = "print(" + closeOpen(statement) + ", sep=\"\")"
+			statement = "print(" + formatting.closeOpen(statement) + ", sep=\"\")"
 		# Variables
 		elif "->" in line or "→" in line:
 			statement = re.split("->|→", line)
@@ -158,15 +73,15 @@ class TIBasicParser(object):
 				if self.basic[index + 1] == "Then":
 					# If/Then statement
 					statement = "if " + line.lstrip("If ")
-					statement = fixEquals(statement)
+					statement = formatting.fixEquals(statement)
 					statement = statement + ":"
 					self.indentIncrease = True
 				elif re.search(r"If.*[^\"]:", line) is not None:
 					# If statement on 1 line
-					statement = ["if " + fixEquals(line.lstrip("If ").split(":", 1)[0]) + ":", "\t" + self.convertLine(index + 1, line.lstrip("If ").split(":", 1)[1])]
+					statement = ["if " + formatting.fixEquals(line.lstrip("If ").split(":", 1)[0]) + ":", "\t" + self.convertLine(index + 1, line.lstrip("If ").split(":", 1)[1])]
 				else:
 					# If statement on 2 lines; no Then
-					statement = ["if " + fixEquals(line.lstrip("If ")) + ":", '\t' + self.convertLine(index + 1, self.basic[index + 1])]
+					statement = ["if " + formatting.fixEquals(line.lstrip("If ")) + ":", '\t' + self.convertLine(index + 1, self.basic[index + 1])]
 
 					if self.basic[index + 2].startswith('End'):
 						self.skipLine = 2 # prioritize ending the if statement
@@ -175,13 +90,13 @@ class TIBasicParser(object):
 			except IndexError:
 				# Last line in file, test for 1 line If statement
 				if re.search(r"If.*[^\"]:", line) is not None:
-					statement = ["if " + fixEquals(line.lstrip("If ").split(":", 1)[0]) + ":", "\t" + self.convertLine(index + 1, line.lstrip("If ").split(":", 1)[1])]
+					statement = ["if " + formatting.fixEquals(line.lstrip("If ").split(":", 1)[0]) + ":", "\t" + self.convertLine(index + 1, line.lstrip("If ").split(":", 1)[1])]
 		elif line == "Then":
 			return None
 		# Elif
 		elif line == "Else" and self.basic[index + 1].startswith("If"):
 			statement = "elif " + self.basic[index + 1].lstrip("If ")
-			statement = fixEquals(statement)
+			statement = formatting.fixEquals(statement)
 			statement = statement + ":"
 			self.indentDecrease = True
 			self.indentIncrease = True
@@ -201,9 +116,9 @@ class TIBasicParser(object):
 			statement = line.split(",")
 			if len(statement) > 1:
 				if "," in statement[1]:
-					statement = statement[1] + " = [toNumber(number) for number in input(" + closeOpen(statement[0][6:]) + ")]"
+					statement = statement[1] + " = [toNumber(number) for number in input(" + formatting.closeOpen(statement[0][6:]) + ")]"
 				else:
-					statement = statement[1] + " = toNumber(input(" + closeOpen(statement[0][6:]) + "))"
+					statement = statement[1] + " = toNumber(input(" + formatting.closeOpen(statement[0][6:]) + "))"
 			else:
 				if statement[0].strip() != "Input":
 					# No input text
@@ -221,11 +136,11 @@ class TIBasicParser(object):
 			self.indentIncrease = True
 		# While loop
 		elif line.startswith("While "):
-			statement = "while " + fixEquals(line[6:]) + ":"
+			statement = "while " + formatting.fixEquals(line[6:]) + ":"
 			self.indentIncrease = True
 		# Repeat loop (tests at bottom)
 		elif line.startswith("Repeat "):
-			statement = ["firstPass = True", "while firstPass == True or not (" + fixEquals(line[7:]) + "):", "\tfirstPass = False"]
+			statement = ["firstPass = True", "while firstPass == True or not (" + formatting.fixEquals(line[7:]) + "):", "\tfirstPass = False"]
 			self.indentIncrease = True
 		# Pause
 		elif line.startswith("Pause"):
@@ -267,7 +182,7 @@ class TIBasicParser(object):
 			self.UTILS["goto"]["enabled"] = True
 		# Output
 		elif line.startswith("Output("):
-			statement = noStringReplace('Output', 'output', [closeOpen(line)])
+			statement = formatting.noStringReplace('Output', 'output', [formatting.closeOpen(line)])
 			self.UTILS["output"]["enabled"] = True
 		# DS<(
 		elif line.startswith("DS<("):
@@ -288,12 +203,12 @@ class TIBasicParser(object):
 			options = []
 			for i in range(0, len(tiMenu), 2):
 				options.extend([tiMenu[i].strip(" \""), tiMenu[i + 1].strip(" \"")])
-			statement = menu(title, options)
+			statement = formatting.menu(title, options)
 			self.UTILS["menu"]["enabled"] = True
 			self.UTILS['clear']['enabled'] = True
 		# Line(
 		elif line.startswith('Line('):
-			statement = closeOpen(line.replace('Line(', 'draw.line('))
+			statement = formatting.closeOpen(line.replace('Line(', 'draw.line('))
 			self.UTILS['draw']['enabled'] = True
 
 		# BackgroundOff
@@ -314,42 +229,42 @@ class TIBasicParser(object):
 
 		# Circle
 		elif line.startswith('Circle('):
-			statement = closeOpen(line.replace('Circle(', 'draw.circle('))
+			statement = formatting.closeOpen(line.replace('Circle(', 'draw.circle('))
 			self.UTILS['draw']['enabled'] = True
 
 		# Text
 		elif line.startswith('Text('):
-			statement = closeOpen(line.replace('Text(', 'draw.text('))
+			statement = formatting.closeOpen(line.replace('Text(', 'draw.text('))
 			self.UTILS['draw']['enabled'] = True
 
 		# Pxl-On
 		elif line.startswith('Pxl-On('):
-			statement = closeOpen(line.replace('Pxl-On(', 'draw.pxlOn('))
+			statement = formatting.closeOpen(line.replace('Pxl-On(', 'draw.pxlOn('))
 			self.UTILS['draw']['enabled'] = True
 
 		# Pxl-Off
 		elif line.startswith('Pxl-Off('):
-			statement = closeOpen(line.replace('Pxl-Off(', 'draw.pxlOff('))
+			statement = formatting.closeOpen(line.replace('Pxl-Off(', 'draw.pxlOff('))
 			self.UTILS['draw']['enabled'] = True
 
 		# pxl-Test
 		elif line.startswith('pxl-Test('):
-			statement = closeOpen(line.replace('pxl-Test(', 'draw.pxlTest('))
+			statement = formatting.closeOpen(line.replace('pxl-Test(', 'draw.pxlTest('))
 			self.UTILS['draw']['enabled'] = True
 
 		# Pt-On
 		elif line.startswith('Pt-On('):
-			statement = closeOpen(line.replace('Pt-On(', 'draw.ptOn('))
+			statement = formatting.closeOpen(line.replace('Pt-On(', 'draw.ptOn('))
 			self.UTILS['draw']['enabled'] = True
 
 		# Pt-Off
 		elif line.startswith('Pt-Off('):
-			statement = closeOpen(line.replace('Pt-Off(', 'draw.ptOff('))
+			statement = formatting.closeOpen(line.replace('Pt-Off(', 'draw.ptOff('))
 			self.UTILS['draw']['enabled'] = True
 
 		# TextColor
 		elif line.startswith('TextColor('):
-			statement = closeOpen(line.replace('TextColor(', 'draw.textColor('))
+			statement = formatting.closeOpen(line.replace('TextColor(', 'draw.textColor('))
 			self.UTILS['draw']['enabled'] = True
 
 		# DispGraph
@@ -372,65 +287,63 @@ class TIBasicParser(object):
 				statement = "# UNKNOWN INDENTIFIER: {}".format(line)
 				logger.warning("Unknown indentifier on line %s", index + 1)
 
-		# Fix things contained within statement
-		# maybe switch to parsing with `ast` for easier detection of strings/non-strings
 		if isinstance(statement, str):
 			statement = [statement]
 
 		if "getKey" in ' '.join(statement):
 			# Replace getKey with getKey() if getKey is not inside of quotes
-			statement = noStringReplace(r'getKey(?!\()+', "getKey()", statement)
+			statement = formatting.noStringReplace(r'getKey(?!\()+', "getKey()", statement)
 			self.UTILS["getKey"]["enabled"] = True
 		if "[theta]" in ' '.join(statement):
 			# Replace [theta] with theta if [theta] is not inside of quotes
-			statement = [item.replace('[theta]', "theta") for item in statement]
+			statement = [item.replace('[theta]', "θ") for item in statement]
 		if "^" in ' '.join(statement):
 			# Convert every ^ not in a string to **
-			statement = noStringReplace(r'\^', '**', statement)
+			statement = formatting.noStringReplace(r'\^', '**', statement)
 		if "–" in ' '.join(statement):
 			# Remove long dash
 			statement = [item.replace('–', "-") for item in statement]
 		if "getTime" in ' '.join(statement):
 			# Replace getTime with getTime() if getTime is not inside of quotes
-			statement = noStringReplace(r'getTime(?!\()+', 'getTime()', statement)
+			statement = formatting.noStringReplace(r'getTime(?!\()+', 'getTime()', statement)
 			self.UTILS["getDateTime"]["enabled"] = True
 		if "getDate" in ' '.join(statement):
 			# Replace getDate with getDate() if getDate is not inside of quotes
-			statement = noStringReplace(r'getDate(?!\()+', 'getDate()', statement)
+			statement = formatting.noStringReplace(r'getDate(?!\()+', 'getDate()', statement)
 			self.UTILS["getDateTime"]["enabled"] = True
 		if "sqrt(" in ' '.join(statement):
 			# Replace sqrt with math.sqrt if sqrt is not inside of quotes
-			statement = noStringReplace('sqrt', 'math.sqrt', statement)
+			statement = formatting.noStringReplace('sqrt', 'math.sqrt', statement)
 			self.UTILS["math"]["enabled"] = True
 		if "toString(" in ' '.join(statement):
 			# Replace toString() with str() if toString() is not inside of quotes
-			statement = noStringReplace(r'toString\(([^\)]+)\)', r'str(\1)', statement)
+			statement = formatting.noStringReplace(r'toString\(([^\)]+)\)', r'str(\1)', statement)
 		if "rand" in ' '.join(statement):
 			# Replace rand with random.random() if rand is not inside of quotes
-			statement = noStringReplace(r'rand(?!\(|I|o)+', 'random.random()', statement)
-			statement = noStringReplace(r'rand\(([0-9])\)', r'[random.random() for _ in range(\1)]', statement)
+			statement = formatting.noStringReplace(r'rand(?!\(|I|o)+', 'random.random()', statement)
+			statement = formatting.noStringReplace(r'rand\(([0-9])\)', r'[random.random() for _ in range(\1)]', statement)
 			self.UTILS['random']['enabled'] = True
 		if 'dayOfWk(' in ' '.join(statement):
 			self.UTILS['getDateTime']['enabled'] = True
 		if 'remainder(' in ' '.join(statement):
-			statement = noStringReplace(r'remainder\(([^\)]+)\)', lambda m: m.group(1).replace(' ', '').split(',')[0] + ' % ' + m.group(1).replace(' ', '').split(',')[1], statement)
+			statement = formatting.noStringReplace(r'remainder\(([^\)]+)\)', lambda m: m.group(1).replace(' ', '').split(',')[0] + ' % ' + m.group(1).replace(' ', '').split(',')[1], statement)
 
 		if 'dim(' in ' '.join(statement):
-			statement = noStringReplace(r'dim\(', 'len(', statement)
+			statement = formatting.noStringReplace(r'dim\(', 'len(', statement)
 
 		if re.search(r'l[1-6]\([0-9A-Za-z]+\)', ' '.join(statement)):
 			# List subscription
 			try:
-				statement = noStringReplace(r'(l[1-6])\(([0-9A-Za-z]+)\)', lambda m: m.group(1) + '[' + str(int(m.group(2)) - 1) + ']', statement)
+				statement = formatting.noStringReplace(r'(l[1-6])\(([0-9A-Za-z]+)\)', lambda m: m.group(1) + '[' + str(int(m.group(2)) - 1) + ']', statement)
 			except ValueError:
-				statement = noStringReplace(r'(l[1-6])\(([0-9A-Za-z]+)\)', lambda m: m.group(1) + '[' + m.group(2) + ']', statement)
+				statement = formatting.noStringReplace(r'(l[1-6])\(([0-9A-Za-z]+)\)', lambda m: m.group(1) + '[' + m.group(2) + ']', statement)
 
 		if 'randInt(' in ' '.join(statement):
 			# Replace randInt with random.randint
-			statement = noStringReplace('randInt', 'random.randint', statement)
+			statement = formatting.noStringReplace('randInt', 'random.randint', statement)
 
 			for i in range(len(statement)):
-				split_statement = parenthesis_split(closeOpen(statement[i]))
+				split_statement = formatting.parenthesis_split(formatting.closeOpen(statement[i]))
 
 				for j in range(len((split_statement))):
 					if 'random.randint' in split_statement[j]:
@@ -442,6 +355,10 @@ class TIBasicParser(object):
 
 			self.UTILS['random']['enabled'] = True
 
+		if self.multiplication:
+			for index, item in enumerate(statement):
+				statement[index] = formatting.toValidEqn(item)
+
 		if isinstance(statement, list) and len(statement) == 1:
 			statement = statement[0]
 
@@ -451,7 +368,7 @@ class TIBasicParser(object):
 		self.pythonCode = []
 
 		self.pythonCode += ["def main():", "\tl1 = l2 = l3 = l4 = l5 = l6 = [None for _ in range(0, 999)] # init lists"]
-		self.pythonCode += ["\tA = B = C = D = E = F = G = H = I = J = K = L = M = N = O = P = Q = R = S = T = U = V = W = X = Y = Z = theta = 0 # init variables"]
+		self.pythonCode += ["\tA = B = C = D = E = F = G = H = I = J = K = L = M = N = O = P = Q = R = S = T = U = V = W = X = Y = Z = θ = 0 # init variables"]
 
 		self.indentLevel = 1
 		# indentIncrease increases the indent on the next line
